@@ -2,21 +2,24 @@
 #include <exception>
 #include <stdexcept>
 #include <string>
-
-#include <locale.h>
+#include <iterator>
+#include <algorithm>
+#include <ios>
+#include <locale>
 #include <fftw3.h>
 
 #include <QSharedMemory>
+#include <QProcessEnvironment>
 #include <QTemporaryFile>
 #include <QDateTime>
-#include <QApplication>
+#include <QLocale>
 #include <QTranslator>
 #include <QRegularExpression>
 #include <QObject>
 #include <QSettings>
-#include <QLibraryInfo>
 #include <QSysInfo>
 #include <QDir>
+#include <QDirIterator>
 #include <QStandardPaths>
 #include <QStringList>
 #include <QLockFile>
@@ -26,11 +29,20 @@
 #include <QSqlDatabase>
 #include <QSqlQuery>
 #include <QSqlError>
+#include <QVariant>
+#include <QByteArray>
+#include <QBitArray>
+#include <QMetaType>
 
+#include "ExceptionCatchingApplication.hpp"
+#include "Logger.hpp"
 #include "revision_utils.hpp"
 #include "MetaDataRegistry.hpp"
+#include "qt_helpers.hpp"
+#include "L10nLoader.hpp"
 #include "SettingsGroup.hpp"
-#include "TraceFile.hpp"
+//#include "TraceFile.hpp"
+#include "WSJTXLogging.hpp"
 #include "MultiSettings.hpp"
 #include "widgets/mainwindow.h"
 #include "commons.h"
@@ -47,6 +59,7 @@ extern "C" {
 
 namespace
 {
+#if QT_VERSION < QT_VERSION_CHECK (5, 15, 0)
   struct RNGSetup
   {
     RNGSetup ()
@@ -56,45 +69,41 @@ namespace
       qsrand (seed);            // this is good for rand() as well
     }
   } seeding;
+#endif
 
-  // We  can't use  the GUI  after QApplication::exit()  is called  so
-  // uncaught exceptions can  get lost on Windows  systems where there
-  // is    no    console    terminal,     so    here    we    override
-  // QApplication::notify() and  wrap the base  class call with  a try
-  // block to catch and display exceptions in a message box.
-  class ExceptionCatchingApplication final
-    : public QApplication
+  void safe_stream_QVariant (boost::log::record_ostream& os, QVariant const& v)
   {
-  public:
-    explicit ExceptionCatchingApplication (int& argc, char * * argv)
-      : QApplication {argc, argv}
-    {
-    }
-    bool notify (QObject * receiver, QEvent * e) override
-    {
-      try
+    switch (static_cast<QMetaType::Type> (v.type ()))
+      {
+      case QMetaType::QByteArray:
+        os << "0x"
+#if QT_VERSION >= QT_VERSION_CHECK (5, 9, 0)
+           << v.toByteArray ().toHex (':').toStdString ()
+#else
+           << v.toByteArray ().toHex ().toStdString ()
+#endif
+          ;
+        break;
+
+      case QMetaType::QBitArray:
         {
-          return QApplication::notify (receiver, e);
+          auto const& bits = v.toBitArray ();
+          os << "0b";
+          for (int i = 0; i < bits.size (); ++ i)
+            {
+              os << (bits[i] ? '1' : '0');
+            }
         }
-      catch (std::exception const& e)
-        {
-          MessageBox::critical_message (nullptr, translate ("main", "Fatal error"), e.what ());
-          throw;
-        }
-      catch (...)
-        {
-          MessageBox::critical_message (nullptr, translate ("main", "Unexpected fatal error"));
-          throw;
-        }
-    }
-  };
+        break;
+
+      default:
+        os << v.toString ();
+      }
+  }
 }
 
 int main(int argc, char *argv[])
 {
-  // ### Add timestamps to all debug messages
-//  qSetMessagePattern ("[%{time yyyyMMdd HH:mm:ss.zzz t} %{if-debug}D%{endif}%{if-info}I%{endif}%{if-warning}W%{endif}%{if-critical}C%{endif}%{if-fatal}F%{endif}] %{message}");
-
   init_random_seed ();
 
   // make the Qt type magic happen
@@ -104,82 +113,76 @@ int main(int argc, char *argv[])
   // Multiple instances communicate with jt9 via this
   QSharedMemory mem_jt9;
 
+  auto const env = QProcessEnvironment::systemEnvironment ();
+
   ExceptionCatchingApplication a(argc, argv);
   try
     {
-      //
-      // Enable i18n
-      //
-      QTranslator translator_from_resources;
-      // Default translations for releases  use translations stored in
-      // the   resources   file    system   under   the   Translations
-      // directory. These are built by the CMake build system from .ts
-      // files in the translations source directory. New languages are
-      // added by  enabling the  UPDATE_TRANSLATIONS CMake  option and
-      // building with the  new language added to  the LANGUAGES CMake
-      // list  variable.  UPDATE_TRANSLATIONS  will preserve  existing
-      // translations  but   should  only  be  set   when  adding  new
-      // languages.  The  resulting .ts  files should be  checked info
-      // source control for translators to access and update.
-      translator_from_resources.load (QLocale::system (), "wsjtx", "_", ":/Translations");
-      a.installTranslator (&translator_from_resources);
+      // LOG_INfO ("+++++++++++++++++++++++++++ Resources ++++++++++++++++++++++++++++");
+      // {
+      //   QDirIterator resources_iter {":/", QDirIterator::Subdirectories};
+      //   while (resources_iter.hasNext ())
+      //     {
+      //       LOG_INFO (resources_iter.next ());
+      //     }
+      // }
+      // LOG_INFO ("--------------------------- Resources ----------------------------");
 
-      QTranslator translator_from_files;
-      // Load  any matching  translation  from  the current  directory
-      // using the locale name. This allows translators to easily test
-      // their translations  by releasing  (lrelease) a .qm  file into
-      // the    current    directory     with    a    suitable    name
-      // (e.g.  wsjtx_en_GB.qm),  then  running   wsjtx  to  view  the
-      // results. Either the system  locale setting or the environment
-      // variable LANG can be used to select the target language.
-      translator_from_files.load (QString {"wsjtx_"} + QLocale::system ().name ());
-      a.installTranslator (&translator_from_files);
+      QLocale locale;              // get the current system locale
 
-      setlocale (LC_NUMERIC, "C"); // ensure number forms are in
-                                   // consistent format, do this after
-                                   // instantiating QApplication so
-                                   // that GUI has correct l18n
+      // Set C/C++ locale used for logging etc.
+#if defined (Q_OS_WIN)
+      std::locale::global (std::locale ("C"));
+#else
+      std::locale::global (std::locale ("en_US.UTF-8"));
+#endif
 
       // Override programs executable basename as application name.
       a.setApplicationName ("WSJT-X");
       a.setApplicationVersion (version ());
 
       QCommandLineParser parser;
-      parser.setApplicationDescription ("\n" PROJECT_SUMMARY_DESCRIPTION);
+      parser.setApplicationDescription ("\n" PROJECT_DESCRIPTION);
       auto help_option = parser.addHelpOption ();
       auto version_option = parser.addVersionOption ();
 
       // support for multiple instances running from a single installation
       QCommandLineOption rig_option (QStringList {} << "r" << "rig-name"
-                                     , a.translate ("main", "Where <rig-name> is for multi-instance support.")
-                                     , a.translate ("main", "rig-name"));
+                                     , "Where <rig-name> is for multi-instance support."
+                                     , "rig-name");
       parser.addOption (rig_option);
 
       // support for start up configuration
       QCommandLineOption cfg_option (QStringList {} << "c" << "config"
-                                     , a.translate ("main", "Where <configuration> is an existing one.")
-                                     , a.translate ("main", "configuration"));
+                                     , "Where <configuration> is an existing one."
+                                     , "configuration");
       parser.addOption (cfg_option);
 
+      // support for UI language override (useful on Windows)
+      QCommandLineOption lang_option (QStringList {} << "l" << "language"
+                                     , "Where <language> is <lang-code>[-<country-code>]."
+                                     , "language");
+      parser.addOption (lang_option);
+
       QCommandLineOption test_option (QStringList {} << "test-mode"
-                                      , a.translate ("main", "Writable files in test location.  Use with caution, for testing only."));
+                                      , "Writable files in test location.  Use with caution, for testing only.");
       parser.addOption (test_option);
 
       if (!parser.parse (a.arguments ()))
         {
-          MessageBox::critical_message (nullptr, a.translate ("main", "Command line error"), parser.errorText ());
+          MessageBox::critical_message (nullptr, "Command line error", parser.errorText ());
           return -1;
         }
       else
         {
           if (parser.isSet (help_option))
             {
-              MessageBox::information_message (nullptr, a.translate ("main", "Command line help"), parser.helpText ());
+              MessageBox::information_message (nullptr, "Command line help", parser.helpText ());
               return 0;
             }
           else if (parser.isSet (version_option))
             {
-              MessageBox::information_message (nullptr, a.translate ("main", "Application version"), a.applicationVersion ());
+              MessageBox::information_message (nullptr, "Application version", a.applicationVersion ());
               return 0;
             }
         }
@@ -195,7 +198,7 @@ int main(int argc, char *argv[])
             {
               if (temp_name.contains (QRegularExpression {R"([\\/,])"}))
                 {
-                  std::cerr << QObject::tr ("Invalid rig name - \\ & / not allowed").toLocal8Bit ().data () << std::endl;
+                  std::cerr << "Invalid rig name - \\ & / not allowed" << std::endl;
                   parser.showHelp (-1);
                 }
                 
@@ -210,7 +213,9 @@ int main(int argc, char *argv[])
           multiple = true;
         }
 
-      // now we have the application name we can open the settings
+      // now we have the application name we can open the logging and settings
+      WSJTXLogging lg;
+      LOG_INFO (program_title (revision ()) << " - Program startup");
       MultiSettings multi_settings {parser.value (cfg_option)};
 
       // find the temporary files path
@@ -226,8 +231,8 @@ int main(int argc, char *argv[])
           if (QLockFile::LockFailedError == instance_lock.error ())
             {
               auto button = MessageBox::query_message (nullptr
-                                                       , a.translate ("main", "Another instance may be running")
-                                                       , a.translate ("main", "try to remove stale lock file?")
+                                                       , "Another instance may be running"
+                                                       , "try to remove stale lock file?"
                                                        , QString {}
                                                        , MessageBox::Yes | MessageBox::Retry | MessageBox::No
                                                        , MessageBox::Yes);
@@ -246,16 +251,12 @@ int main(int argc, char *argv[])
             }
         }
 
-#if WSJT_QDEBUG_TO_FILE
-      // Open a trace file
-      TraceFile trace_file {temp_dir.absoluteFilePath (a.applicationName () + "_trace.log")};
-      qSetMessagePattern ("[%{time yyyyMMdd HH:mm:ss.zzz t} %{if-debug}D%{endif}%{if-info}I%{endif}%{if-warning}W%{endif}%{if-critical}C%{endif}%{if-fatal}F%{endif}] %{file}:%{line} - %{message}");
-      qDebug () << program_title (revision ()) + " - Program startup";
-#endif
+      // load UI translations
+      L10nLoader l10n {&a, locale, parser.value (lang_option)};
 
       // Create a unique writeable temporary directory in a suitable location
       bool temp_ok {false};
-      QString unique_directory {QApplication::applicationName ()};
+      QString unique_directory {ExceptionCatchingApplication::applicationName ()};
       do
         {
           if (!temp_dir.mkpath (unique_directory)
@@ -327,44 +328,90 @@ int main(int argc, char *argv[])
       db.exec ("PRAGMA locking_mode=EXCLUSIVE");
 
       int result;
+      auto const& original_style_sheet = a.styleSheet ();
       do
         {
-#if WSJT_QDEBUG_TO_FILE
-          // announce to trace file and dump settings
-          qDebug () << "++++++++++++++++++++++++++++ Settings ++++++++++++++++++++++++++++";
-          for (auto const& key: multi_settings.settings ()->allKeys ())
+          // dump settings
+          auto sys_lg = sys::get ();
+          if (auto rec = sys_lg.open_record
+              (
+               boost::log::keywords::severity = boost::log::trivial::trace)
+              )
             {
-              auto const& value = multi_settings.settings ()->value (key);
-              if (value.canConvert<QVariantList> ())
+              boost::log::record_ostream strm (rec);
+              strm << "++++++++++++++++++++++++++++ Settings ++++++++++++++++++++++++++++\n";
+              for (auto const& key: multi_settings.settings ()->allKeys ())
                 {
-                  auto const sequence = value.value<QSequentialIterable> ();
-                  qDebug ().nospace () << key << ": ";
-                  for (auto const& item: sequence)
+                  if (!key.contains (QRegularExpression {"^MultiSettings/[^/]*/"}))
                     {
-                      qDebug ().nospace () << '\t' << item;
+                      auto const& value = multi_settings.settings ()->value (key);
+                      if (value.canConvert<QVariantList> ())
+                        {
+                          auto const sequence = value.value<QSequentialIterable> ();
+                          strm << key << ":\n";
+                          for (auto const& item: sequence)
+                            {
+                              strm << "\t";
+                              safe_stream_QVariant (strm, item);
+                              strm << '\n';
+                            }
+                        }
+                      else
+                        {
+                          strm << key << ": ";
+                          safe_stream_QVariant (strm, value);
+                          strm << '\n';
+                        }
                     }
                 }
-              else
-                {
-                  qDebug ().nospace () << key << ": " << value;
-                }
+              strm << "---------------------------- Settings ----------------------------\n";
+              strm.flush ();
+              sys_lg.push_record (boost::move (rec));
             }
-          qDebug () << "---------------------------- Settings ----------------------------";
-#endif
 
           // Create and initialize shared memory segment
           // Multiple instances: use rig_name as shared memory key
           mem_jt9.setKey(a.applicationName ());
 
-          if(!mem_jt9.attach()) {
-            if (!mem_jt9.create(sizeof(struct dec_data))) {
-              splash.hide ();
-              MessageBox::critical_message (nullptr, a.translate ("main", "Shared memory error"),
-                                            a.translate ("main", "Unable to create shared memory segment"));
-              throw std::runtime_error {"Shared memory error"};
+          // try and shut down any orphaned jt9 process
+          for (int i = 3; i; --i) // three tries to close old jt9
+            {
+              if (mem_jt9.attach ()) // shared memory presence implies
+                                     // orphaned jt9 sub-process
+                {
+                  dec_data_t * dd = reinterpret_cast<dec_data_t *> (mem_jt9.data());
+                  mem_jt9.lock ();
+                  dd->ipc[1] = 999; // tell jt9 to shut down
+                  mem_jt9.unlock ();
+                  mem_jt9.detach (); // start again
+                }
+              else
+                {
+                  break;        // good to go
+                }
+              QThread::sleep (1); // wait for jt9 to end
             }
-          }
+          if (!mem_jt9.attach ())
+            {
+              if (!mem_jt9.create (sizeof (dec_data)))
+              {
+                splash.hide ();
+                MessageBox::critical_message (nullptr, a.translate ("main", "Shared memory error"),
+                                              a.translate ("main", "Unable to create shared memory segment"));
+                throw std::runtime_error {"Shared memory error"};
+              }
+              LOG_INFO ("shmem size: " << mem_jt9.size ());
+            }
+          else
+            {
+              splash.hide ();
+              MessageBox::critical_message (nullptr, a.translate ("main", "Sub-process error"),
+                                            a.translate ("main", "Failed to close orphaned jt9 process"));
+              throw std::runtime_error {"Sub-process error"};
+            }
+          mem_jt9.lock ();
           memset(mem_jt9.data(),0,sizeof(struct dec_data)); //Zero all decoding params in shared memory
+          mem_jt9.unlock ();
 
           unsigned downSampleFactor;
           {
@@ -384,11 +431,14 @@ int main(int argc, char *argv[])
           }
 
           // run the application UI
-          MainWindow w(temp_dir, multiple, &multi_settings, &mem_jt9, downSampleFactor, &splash);
+          MainWindow w(temp_dir, multiple, &multi_settings, &mem_jt9, downSampleFactor, &splash, env);
           w.show();
           splash.raise ();
           QObject::connect (&a, SIGNAL (lastWindowClosed()), &a, SLOT (quit()));
           result = a.exec();
+
+          // ensure config switches start with the right style sheet
+          a.setStyleSheet (original_style_sheet);
         }
       while (!result && !multi_settings.exit ());
 
@@ -409,12 +459,12 @@ int main(int argc, char *argv[])
     }
   catch (std::exception const& e)
     {
-      MessageBox::critical_message (nullptr, QApplication::translate ("main", "Fatal error"), e.what ());
+      MessageBox::critical_message (nullptr, "Fatal error", e.what ());
       std::cerr << "Error: " << e.what () << '\n';
     }
   catch (...)
     {
-      MessageBox::critical_message (nullptr, QApplication::translate ("main", "Unexpected fatal error"));
+      MessageBox::critical_message (nullptr, "Unexpected fatal error");
       std::cerr << "Unexpected fatal error\n";
       throw;			// hoping the runtime might tell us more about the exception
     }
