@@ -13,6 +13,7 @@
 #include <QVector>
 #include <QStringList>
 #include <QMimeData>
+#include <QTextStream>
 #include <QDataStream>
 #include <QByteArray>
 #include <QDebugStateSaver>
@@ -46,16 +47,20 @@ namespace
       {20000000, Modes::FreqCal, IARURegions::ALL},
       
       {136000, Modes::WSPR, IARURegions::ALL},
-      {136130, Modes::JT65, IARURegions::ALL},
-      {136130, Modes::JT9, IARURegions::ALL},
+      {136000, Modes::FST4, IARURegions::ALL},
+      {136000, Modes::FST4W, IARURegions::ALL},
+      {136000, Modes::JT9, IARURegions::ALL},
 
-      {474200, Modes::JT65, IARURegions::ALL},
       {474200, Modes::JT9, IARURegions::ALL},
+      {474200, Modes::FST4, IARURegions::ALL},
       {474200, Modes::WSPR, IARURegions::ALL},
-      
+      {474200, Modes::FST4W, IARURegions::ALL},
+
       {1836600, Modes::WSPR, IARURegions::ALL},
+      {1836800, Modes::FST4W, IARURegions::ALL},
       {1838000, Modes::JT65, IARURegions::ALL}, // squeezed allocations
       {1839000, Modes::JT9, IARURegions::ALL},
+      {1839000, Modes::FST4, IARURegions::ALL},
       {1840000, Modes::FT8, IARURegions::ALL},
 
       // Band plans (all USB dial unless stated otherwise)
@@ -98,18 +103,18 @@ namespace
       //     7050 - 7060 DM WB(<2700Hz) with 7050 - 7053 ACDS
       //
       //     7040         PSK31
-      //     7043.25      OLIVIA, Contestia, etc. (main QRQ)
+      //     7043.25      OLIVIA, Contestia, etc. (main QRG)
       //     7070         PSK31
-      //     7073.25      OLIVIA, Contestia, etc. (main QRQ)
+      //     7073.25      OLIVIA, Contestia, etc. (main QRG)
       //     7090         LSB QRP CoA
       //
       // R2: 7040 - 7050 DM NB(<500Hz)  with 7047 - 7050 ACDS
       //     7050 - 7053 DM WB(<2700Hz) ACDS shared with all modes
       //
       //     7040         RTTY DX
-      //     7043.25      OLIVIA, Contestia, etc. (main QRQ)
+      //     7043.25      OLIVIA, Contestia, etc. (main QRG)
       //     7070         PSK31 (also LSB EMCOMM)
-      //     7073.25      OLIVIA, Contestia, etc. (main QRQ)
+      //     7073.25      OLIVIA, Contestia, etc. (main QRG)
       //     7080 - 7125  RTTY/Data
       //     7090         LSB QRP CoA
       //
@@ -117,7 +122,7 @@ namespace
       //
       //     7030 - 7100 DM WB(<3000Hz) JA 7045 - 7100 shared with all modes
       //
-      //     7026.25      OLIVIA, Contestia, etc. (main QRQ)
+      //     7026.25      OLIVIA, Contestia, etc. (main QRG)
       //     7035         PSK31
       //     7050         JA LSB EMCOMM
       //     7090         LSB QRP CoA
@@ -138,7 +143,7 @@ namespace
       //     10139.25       OLIVIA, Contestia, etc.
       //     10142          PSK31
       //     10142.25       OLIVIA, Contestia, etc.
-      //     10143.25       OLIVIA, Contestia, etc. (main QRQ)
+      //     10143.25       OLIVIA, Contestia, etc. (main QRG)
       //
       // R2: 10130 - 10140 DM NB(<500Hz)  shared with ACDS
       //     10140 - 10150 DM WB(<2700Hz)
@@ -148,14 +153,14 @@ namespace
       //     10140 - 10150  Packet
       //     10142          PSK31
       //     10142.25       OLIVIA, Contestia, etc.
-      //     10143.25       OLIVIA, Contestia, etc. (main QRQ)
+      //     10143.25       OLIVIA, Contestia, etc. (main QRG)
       // 
       // R3: 10130 - 10150 DM NB(<2000Hz)
       //
       //     10139.25       OLIVIA, Contestia, etc.
       //     10142          PSK31
       //     10142.25       OLIVIA, Contestia, etc.
-      //     10143.25       OLIVIA, Contestia, etc. (main QRQ)
+      //     10143.25       OLIVIA, Contestia, etc. (main QRG)
       //
       {10136000, Modes::FT8, IARURegions::ALL},
       {10138000, Modes::JT65, IARURegions::ALL},
@@ -238,7 +243,7 @@ namespace
       {18104000, Modes::JT9, IARURegions::ALL},
       {18104000, Modes::FT4, IARURegions::ALL}, // provisional
       {18104600, Modes::WSPR, IARURegions::ALL},
-      
+
       {21074000, Modes::FT8, IARURegions::ALL},
       {21076000, Modes::JT65, IARURegions::ALL},
       {21078000, Modes::JT9, IARURegions::ALL},
@@ -250,7 +255,7 @@ namespace
       {24919000, Modes::JT9, IARURegions::ALL},
       {24919000, Modes::FT4, IARURegions::ALL}, // provisional
       {24924600, Modes::WSPR, IARURegions::ALL},
-      
+
       {28074000, Modes::FT8, IARURegions::ALL},
       {28076000, Modes::JT65, IARURegions::ALL},
       {28078000, Modes::JT9, IARURegions::ALL},
@@ -343,14 +348,21 @@ namespace
 QDebug operator << (QDebug debug, FrequencyList_v2::Item const& item)
 {
   QDebugStateSaver saver {debug};
-  debug.nospace () << "FrequencyItem("
-                   << item.frequency_ << ", "
-                   << item.region_ << ", "
-                   << item.mode_ << ')';
-  return debug;
+  return debug.nospace () << item.toString ();
 }
 #endif
 
+QString FrequencyList_v2::Item::toString () const
+{
+  QString string;
+  QTextStream qts {&string};
+  qts << "FrequencyItem("
+      << Radio::frequency_MHz_string (frequency_) << ", "
+      << IARURegions::name (region_) << ", "
+      << Modes::name (mode_) << ')';
+  return string;
+}
+
 QDataStream& operator << (QDataStream& os, FrequencyList_v2::Item const& item)
 {
   return os << item.frequency_
@@ -568,6 +580,7 @@ auto FrequencyList_v2::impl::frequency_list (FrequencyItems frequency_list) -> F
   return frequency_list;
 }
 
+// add a frequency returning the new model index
 QModelIndex FrequencyList_v2::impl::add (Item f)
 {
   // Any Frequency that isn't in the list may be added
