@@ -13,10 +13,10 @@
 #include <QListIterator>
 #include <QRegularExpression>
 #include <QScrollBar>
-#include <QDebug>
 
 #include "Configuration.hpp"
-#include "LotWUsers.hpp"
+#include "Decoder/decodedtext.h"
+#include "Network/LotWUsers.hpp"
 #include "models/DecodeHighlightingModel.hpp"
 #include "logbook/logbook.h"
 
@@ -126,8 +126,6 @@ namespace
 void DisplayText::appendText(QString const& text, QColor bg, QColor fg
                              , QString const& call1, QString const& call2)
 {
-//  qDebug () << "DisplayText::appendText: text:" << text << "Nbsp pos:" << text.indexOf (QChar::Nbsp);
-
   auto cursor = textCursor ();
   cursor.movePosition (QTextCursor::End);
   auto block_format = cursor.blockFormat ();
@@ -244,10 +242,9 @@ void DisplayText::new_period ()
 
 QString DisplayText::appendWorkedB4 (QString message, QString call, QString const& grid,
                                      QColor * bg, QColor * fg, LogBook const& logBook,
-                                     QString const& currentBand, QString const& currentMode)
+                                     QString const& currentBand, QString const& currentMode,
+                                     QString extra)
 {
-  // allow for seconds
-  int padding {message.indexOf (" ") > 4 ? 2 : 0};
   QString countryName;
   bool callB4;
   bool callB4onBand;
@@ -281,7 +278,6 @@ QString DisplayText::appendWorkedB4 (QString message, QString call, QString cons
   }
 
   message = message.trimmed ();
-  QString appendage;
 
   highlight_types types;
   // no shortcuts here as some types may be disabled
@@ -332,20 +328,20 @@ QString DisplayText::appendWorkedB4 (QString message, QString call, QString cons
     {
     case Highlight::Continent:
     case Highlight::ContinentBand:
-      appendage = AD1CCty::continent (looked_up.continent);
+      extra += AD1CCty::continent (looked_up.continent);
       break;
     case Highlight::CQZone:
     case Highlight::CQZoneBand:
-      appendage = QString {"CQ Zone %1"}.arg (looked_up.CQ_zone);
+      extra += QString {"CQ Zone %1"}.arg (looked_up.CQ_zone);
       break;
     case Highlight::ITUZone:
     case Highlight::ITUZoneBand:
-      appendage = QString {"ITU Zone %1"}.arg (looked_up.ITU_zone);
+      extra += QString {"ITU Zone %1"}.arg (looked_up.ITU_zone);
       break;
     default:
       if (m_bPrincipalPrefix)
         {
-          appendage = looked_up.primary_prefix;
+          extra += looked_up.primary_prefix;
         }
       else
         {
@@ -371,26 +367,38 @@ QString DisplayText::appendWorkedB4 (QString message, QString call, QString cons
           countryName.replace ("European", "EU");
           countryName.replace ("African", "AF");
 
-          appendage += countryName;
+          extra += countryName;
         }
     }
     m_CQPriority=DecodeHighlightingModel::highlight_name(top_highlight);
 
-  // use a nbsp to save the start of appended text so we can find
-  // it again later, align appended data at a fixed column if
-  // there is space otherwise let it float to the right
-  int space_count {40 + padding - message.size ()};
-  if (space_count > 0) {
-    message += QString {space_count, QChar {' '}};
-  }
-  message += QChar::Nbsp + appendage;
+    return leftJustifyAppendage (message, extra);
+}
+
+QString DisplayText::leftJustifyAppendage (QString message, QString const& appendage) const
+{
+  if (appendage.size ())
+    {
+      // allow for seconds
+      int padding {message.indexOf (" ") > 4 ? 2 : 0};
+
+      // use a nbsp to save the start of appended text so we can find
+      // it again later, align appended data at a fixed column if
+      // there is space otherwise let it float to the right
+      int space_count {40 + padding - message.size ()};
+      if (space_count > 0) {
+        message += QString {space_count, QChar {' '}};
+      }
+      message += QChar::Nbsp + appendage;
+    }
   return message;
 }
 
 void DisplayText::displayDecodedText(DecodedText const& decodedText, QString const& myCall,
                                      QString const& mode,
                                      bool displayDXCCEntity, LogBook const& logBook,
-                                     QString const& currentBand, bool ppfx, bool bCQonly)
+                                     QString const& currentBand, bool ppfx, bool bCQonly,
+                                     bool haveFSpread, float fSpread)
 {
   m_bPrincipalPrefix=ppfx;
   QColor bg;
@@ -423,7 +431,18 @@ void DisplayText::displayDecodedText(DecodedText const& decodedText, QString con
   decodedText.deCallAndGrid (/*out*/ dxCall, dxGrid);
   QRegularExpression grid_regexp {"\\A(?![Rr]{2}73)[A-Ra-r]{2}[0-9]{2}([A-Xa-x]{2}){0,1}\\z"};
   if(!dxGrid.contains(grid_regexp)) dxGrid="";
-  message = message.left (message.indexOf (QChar::Nbsp)); // strip appended info
+  message = message.left (message.indexOf (QChar::Nbsp)).trimmed (); // strip appended info
+  QString extra;
+  if (haveFSpread)
+    {
+      extra += QString {"%1"}.arg (fSpread, 5, 'f', fSpread < 0.95 ? 3 : 2) + QChar {' '};
+    }
+  auto ap_pos = message.lastIndexOf (QRegularExpression {R"((?:\?\s)?a[0-9]$)"});
+  if (ap_pos >= 0)
+    {
+      extra += message.mid (ap_pos) + QChar {' '};
+      message = message.left (ap_pos).trimmed ();
+    }
   m_CQPriority="";
   if (CQcall)
     {
@@ -437,10 +456,11 @@ void DisplayText::displayDecodedText(DecodedText const& decodedText, QString con
               currentMode = decodedText.isJT65 () ? "JT65" : "JT9";
             }
           message = appendWorkedB4 (message, decodedText.CQersCall(), dxGrid, &bg, &fg
-                                    , logBook, currentBand, currentMode);
+                                    , logBook, currentBand, currentMode, extra);
         }
       else
         {
+          message = leftJustifyAppendage (message, extra);
           highlight_types types {Highlight::CQ};
           if (m_config && m_config->lotw_users ().user (decodedText.CQersCall()))
             {
@@ -449,12 +469,17 @@ void DisplayText::displayDecodedText(DecodedText const& decodedText, QString con
           set_colours (m_config, &bg, &fg, types);
         }
     }
+  else
+    {
+      message = leftJustifyAppendage (message, extra);
+    }
 
   appendText (message.trimmed (), bg, fg, decodedText.call (), dxCall);
 }
 
 
-void DisplayText::displayTransmittedText(QString text, QString modeTx, qint32 txFreq,bool bFastMode)
+void DisplayText::displayTransmittedText(QString text, QString modeTx, qint32 txFreq,
+                                         bool bFastMode, double TRperiod)
 {
     QString t1=" @  ";
     if(modeTx=="FT4") t1=" +  ";
@@ -462,10 +487,11 @@ void DisplayText::displayTransmittedText(QString text, QString modeTx, qint32 tx
     if(modeTx=="JT4") t1=" $  ";
     if(modeTx=="JT65") t1=" #  ";
     if(modeTx=="MSK144") t1=" &  ";
+    if(modeTx=="FST4") t1=" `  ";
     QString t2;
-    t2.sprintf("%4d",txFreq);
+    t2 = t2.asprintf("%4d",txFreq);
     QString t;
-    if(bFastMode or modeTx=="FT8" or modeTx=="FT4") {
+    if(bFastMode or modeTx=="FT8" or modeTx=="FT4" or (TRperiod<60)) {
       t = QDateTime::currentDateTimeUtc().toString("hhmmss") + \
         "  Tx      " + t2 + t1 + text;
     } else if(modeTx.mid(0,6)=="FT8fox") {
@@ -497,60 +523,98 @@ namespace
 {
   void update_selection (QTextCursor& cursor, QColor const& bg, QColor const& fg)
   {
-    if (!cursor.isNull ())
+    QTextCharFormat format {cursor.charFormat ()};
+    if (bg.isValid ())
       {
-        QTextCharFormat format {cursor.charFormat ()};
-        if (bg.isValid ())
-          {
-            format.setBackground (bg);
-          }
-        else
-          {
-            format.clearBackground ();
-          }
-        if (fg.isValid ())
-          {
-            format.setForeground (fg);
-          }
-        else
-          {
-            format.clearForeground ();
-          }
-        cursor.mergeCharFormat (format);
+        format.setBackground (bg);
       }
+    else
+      {
+        format.clearBackground ();
+      }
+    if (fg.isValid ())
+      {
+        format.setForeground (fg);
+      }
+    else
+      {
+        format.clearForeground ();
+      }
+    cursor.mergeCharFormat (format);
   }
 
   void reset_selection (QTextCursor& cursor)
   {
-    if (!cursor.isNull ())
+    // restore previous text format, we rely on the text
+    // char format at he start of the selection being the
+    // old one which should be the case
+    auto c2 = cursor;
+    c2.setPosition (c2.selectionStart ());
+    cursor.setCharFormat (c2.charFormat ());
+  }
+}
+
+namespace
+{
+  QString get_timestamp (QTextCursor& cursor)
+  {
+    QString timestamp;
+    if (cursor.movePosition (QTextCursor::PreviousCharacter)
+        && cursor.movePosition (QTextCursor::StartOfLine)
+        && cursor.movePosition (QTextCursor::EndOfWord, QTextCursor::KeepAnchor)
+        && cursor.hasSelection ())
       {
-        // restore previous text format, we rely on the text
-        // char format at he start of the selection being the
-        // old one which should be the case
-        auto c2 = cursor;
-        c2.setPosition (c2.selectionStart ());
-        cursor.setCharFormat (c2.charFormat ());
+        timestamp = cursor.selectedText ();
+        cursor.movePosition (QTextCursor::StartOfLine);
       }
+    return timestamp;
   }
 }
 
 void DisplayText::highlight_callsign (QString const& callsign, QColor const& bg,
-                                      QColor const& fg, bool last_only)
+                                      QColor const& fg, bool last_period_only)
 {
+  if (!callsign.size ())
+    {
+      return;
+    }
+  auto regexp = callsign;
+  // allow for hashed callsigns and escape any regexp metacharacters
+  QRegularExpression target {QString {"<?"}
+                             + regexp.replace (QLatin1Char {'+'}, QLatin1String {"\\+"})
+                                 .replace (QLatin1Char {'.'}, QLatin1String {"\\."})
+                                 .replace (QLatin1Char {'?'}, QLatin1String {"\\?"})
+                             + QString {">?"}
+                             , QRegularExpression::DontCaptureOption};
   QTextCharFormat old_format {currentCharFormat ()};
   QTextCursor cursor {document ()};
-  if (last_only)
+  if (last_period_only)
     {
+      // highlight each instance of the given callsign (word) in the
+      // current period
       cursor.movePosition (QTextCursor::End);
-      cursor = document ()->find (callsign, cursor
-                                  , QTextDocument::FindBackward | QTextDocument::FindWholeWords);
-      if (bg.isValid () || fg.isValid ())
+      QTextCursor period_start {cursor};
+      QTextCursor prior {cursor};
+      auto period_timestamp = get_timestamp (period_start);
+      while (period_timestamp.size () && period_timestamp == get_timestamp (prior))
         {
-          update_selection (cursor, bg, fg);
+          period_start = prior;
         }
-      else
+      cursor = period_start;
+      while (!cursor.isNull ())
         {
-          reset_selection (cursor);
+          cursor = document ()->find (target, cursor, QTextDocument::FindWholeWords);
+          if (!cursor.isNull () && cursor.hasSelection ())
+            {
+              if (bg.isValid () || fg.isValid ())
+                {
+                  update_selection (cursor, bg, fg);
+                }
+              else
+                {
+                  reset_selection (cursor);
+                }
+            }
         }
     }
   else
@@ -569,18 +633,27 @@ void DisplayText::highlight_callsign (QString const& callsign, QColor const& bg,
             }
           while (!cursor.isNull ())
             {
-              cursor = document ()->find (callsign, cursor, QTextDocument::FindWholeWords);
-              update_selection (cursor, bg, fg);
+              cursor = document ()->find (target, cursor, QTextDocument::FindWholeWords);
+              if (!cursor.isNull () && cursor.hasSelection ())
+                {
+                  update_selection (cursor, bg, fg);
+                }
             }
         }
-      else if (pos != highlighted_calls_.end ())
+      else
         {
-          highlighted_calls_.erase (pos);
+          if (pos != highlighted_calls_.end ())
+            {
+              highlighted_calls_.erase (pos);
+            }
           QTextCursor cursor {document ()};
           while (!cursor.isNull ())
             {
-              cursor = document ()->find (callsign, cursor, QTextDocument::FindWholeWords);
-              reset_selection (cursor);
+              cursor = document ()->find (target, cursor, QTextDocument::FindWholeWords);
+              if (!cursor.isNull () && cursor.hasSelection ())
+                {
+                  reset_selection (cursor);
+                }
             }
         }
     }
