@@ -49,6 +49,8 @@ public:
   void   drawRed(int ia, int ib);
   void   setVHF(bool bVHF);
   void   setRedFile(QString fRed);
+  void   setFST4_FreqRange(int fLow,int fHigh);
+  void   setSingleDecode(bool b);
 
 signals:
   void freezeDecode2(int n);
@@ -68,7 +70,7 @@ protected:
 private slots:
   void on_waterfallAvgSpinBox_valueChanged(int arg1);
   void on_bppSpinBox_valueChanged(int arg1);
-  void on_spec2dComboBox_currentIndexChanged(const QString &arg1);
+  void on_spec2dComboBox_currentIndexChanged(int);
   void on_fSplitSpinBox_valueChanged(int n);
   void on_fStartSpinBox_valueChanged(int n);
   void on_paletteComboBox_activated(const QString &palette);
@@ -115,6 +117,8 @@ private:
   QString m_mode;
   QString m_modeTx;
   QString m_waterfallPalette;  
+  float   m_swide[MAX_SCREENSIZE];
+  QString m_user_defined;
 };
 
 #endif // WIDEGRAPH_H
